﻿using ProjectGasMachine.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGasMachine.Drawnings;

public static class ExtentionDrawningGas
{
    /// <summary>
    /// Разделитель для записи информации по объекту в файл
    /// </summary>
    private static readonly string _separatorForObject = ":";

    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawningMachine? CreateDrawningMachine(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityGas? machine = EntityMachine.CreateEntityMachine(strs);
        if (machine != null)
        {
            return new DrawningGasMachine(machine);
        }

        machine = EntityGas.CreateEntityGas(strs);
        if (machine != null)
        {
            return new DrawningMachine(machine);
        }

        return null;
    }

    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningMachine">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawningMachine drawningMachine)
    {
        string[]? array = drawningMachine?.EntityGas?.GetStringRepresentation();

        if (array == null)
        {
            return string.Empty;
        }

        return string.Join(_separatorForObject, array);
    }
}
