﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectGasMachine.Entities;

/// <summary>
/// Класс-сущность "машина для перевозки газа"
/// </summary>
public class EntityGas
{
    /// <summary>
    /// Скорость
    /// </summary>
    public int Speed { get; private set; }
    /// <summary>
    /// Вес 
    /// </summary>
    public double Weight { get; private set; }
    /// <summary>
    /// Основной цвет 
    /// </summary>
    
    public Color BodyColor { get; private set; }
    /// <summary>
    /// Шаг перемещения машины
    /// </summary>
    public double Step => Speed * 100 / Weight;

    /// <summary>
    /// Конструктор сущности
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес машины</param>
    /// <param name="bodyColor">Основной цвет</param>
    public EntityGas(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
  
    }

    /// <summary>
    /// Смена основного цвета
    /// </summary>
    /// <param name="newColor"></param>
    public void BodyColorChange(Color newColor)
    {
        BodyColor = newColor;
    }


    /// <summary>
    /// Получение строк со значениями свойств объекта класса-сущности
    /// </summary>
    /// <returns></returns>
    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityGas), Speed.ToString(), Weight.ToString(), BodyColor.Name };
    }

    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityGas? CreateEntityGas(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityGas))
        {
            return null;
        }

        return new EntityGas(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }
}
