﻿namespace ProjectGasMachine.Entities;
/// <summary>
/// Класс-сущность "бензовоз"
/// </summary>
public class EntityMachine : EntityGas
{
    
    public Color AdditionalColor { get; private set; }
    /// <summary>
    /// Признак (опция) наличия бака для бензина
    /// </summary>
    public bool Gas { get; private set; }
    /// <summary>
    /// Признак (опция) наличия маяка
    /// </summary>
    public bool Beacon { get; private set; }
    /// <summary>
    /// 
    /// </summary>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="gas">Признак наличия бака для бензина</param>
    /// <param name="beacon">Признак наличия маяка</param>

    public EntityMachine(int speed, double weight, Color bodyColor, Color additionalColor, bool gas, bool beacon) : base(speed, weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        Gas = gas;
        Beacon = beacon;
    }

    /// <summary>
    /// Смена дополнительного цвета
    /// </summary>
    /// <param name="newColor"></param>
    public void AdditionalColorChange(Color newColor)
    {
        AdditionalColor = newColor;
    }

    public override string[] GetStringRepresentation()
    {
        return new[]
        {
            nameof(EntityMachine),
            Speed.ToString(),
            Weight.ToString(),
            BodyColor.Name,
            AdditionalColor.Name,
            Gas.ToString(),
            Beacon.ToString(),
        };
    }

    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityMachine? CreateEntityMachine(string[] strs)
    {
        if (strs.Length != 7 || strs[0] != nameof(EntityMachine))
        {
            return null;
        }

        return new EntityMachine(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]),
            Color.FromName(strs[4]), Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]));
    }
}
