﻿using ProjectGasMachine.Drawnings;
using ProjectGasMachine.Entities;

namespace ProjectGasMachine;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormGasConfig : Form
{
    /// <summary>
    /// Объект - прорисовка машины
    /// </summary>
    private DrawningMachine? _machine;

    /// <summary>
    /// событие для передачи объекта
    /// </summary>
    private event Action<DrawningMachine>? MachineDelegate;

    /// <summary>
    /// Привязка внешнего метода к событию
    /// </summary>
    /// <param name="machineDelegate"></param>
    public void AddEvent(Action<DrawningMachine>? machineDelegate)
    {
        MachineDelegate += machineDelegate;
    }

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormGasConfig()
    {
        InitializeComponent();

        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        buttonCancel.Click += (sender, e) => Close();
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _machine?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _machine?.SetPosition(5, 5);
        _machine?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаем информацию при нажатии на Label(Объект)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации(Объект) (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действия при приеме перетаскиваемой информации(Объект)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _machine = new DrawningMachine((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _machine = new DrawningGasMachine((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                Color.Black, checkBoxGas.Checked, checkBoxBeacon.Checked);
                break;
        }

        DrawObject();
    }
    /// <summary>
    ///  Передаем информацию при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        // TODO отправка цвета в Drag&Drop +
        (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor ?? Color.Black, DragDropEffects.Move | DragDropEffects.Copy);
    }
    /// <summary>
    /// Проверка получаемой информации(Основной цвет) (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(typeof(Color)) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действия при приеме перетаскиваемой информации(Основной цвет)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        _machine?.EntityGas?.BodyColorChange((Color)e.Data?.GetData(typeof(Color)));
        DrawObject();
    }

    /// <summary>
    /// Проверка получаемой информации(Доп. цвет) (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(typeof(Color)) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действия при приеме перетаскиваемой информации(Доп. цвет)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void LabelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_machine?.EntityGas is EntityMachine _gasmachine)
        {
            _gasmachine.AdditionalColorChange((Color)e.Data?.GetData(typeof(Color)));
        }
        DrawObject();
    }


    /// <summary>
    /// Передача объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_machine != null)
        {
            MachineDelegate?.Invoke(_machine);
            Close();
        }
    }

    

}
