﻿using ProjectGasMachine.Drawnings;
using ProjectGasMachine.MovementStrategy;


namespace ProjectGasMachine;
public partial class FormGasMachine : Form
{
    /// <summary>
    /// поле-объект для прорисовки объекта
    /// </summary>
    private DrawningMachine? _drawningMachine;

    /// <summary>
    /// стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    public DrawningMachine SetMachine 
    {
        set 
        {
            _drawningMachine = value;
            _drawningMachine.SetPictureSize(pictureBoxGasMachine.Height, pictureBoxGasMachine.Width);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// конструктор формы
    /// </summary>
    public FormGasMachine()
    {
        InitializeComponent();
        _strategy = null;
    }


    /// <summary>
    /// метод прорисовки машины
    /// </summary>
    private void Draw()
    {
        if (_drawningMachine == null)
        {
            return;
        }


        Bitmap bmp = new(pictureBoxGasMachine.Width, pictureBoxGasMachine.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningMachine.DrawTransport(gr);
        pictureBoxGasMachine.Image = bmp;
    }



    /// <summary>
    /// перемещение объекта по форме
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningMachine == null)
        {
            return;
        }
        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningMachine.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningMachine.MoveTransport(DirectionType.Down);
                break;
            case "buttonRight":
                result = _drawningMachine.MoveTransport(DirectionType.Right);
                break;
            case "buttonLeft":
                result = _drawningMachine.MoveTransport(DirectionType.Left);
                break;
        }
        if (result)
        {
            Draw();
        }
    }

    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningMachine == null) 
        {
            return;
        }

        if (comboBoxStrategy.Enabled) 
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };

            if (_strategy == null) 
            {
                return;
            }

            _strategy.SetData(new MoveableMachine(_drawningMachine), pictureBoxGasMachine.Width, pictureBoxGasMachine.Height);     
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish) 
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }




    }
}

